﻿/**
 * Copyright 2011 ISO NewEngland Inc.
 * 
 * Caveat:
 * This example program is intended to serve only as working example of how to access eMarket MUI Webservices through Java and C# Clients. 
 * It is not supported nor intended for production use. 
 * Also, it is not intended to serve as a tool for end users. 
 * 
 * Furthermore, without modification, this example code will not work with eFTR, EES, OutageScheduler, 
 * or the Settlements Market System. The example does not implement behavior critical to the interaction with the  
 * aforementioned applications including: multi-part MIME, cookie handling, and capturing responses to a file. 
 * 
 * It is expected that a competent developer of moderate skill and knowledge of Java and/or C# and the HTTP protocol 
 * would be able to use these examples as a guide for developing more sophisticated clients to interact  
 * with the eMarket MUI WebServices.
 */

using System;                                       //Console and Exception
using System.Collections.Generic;                   //Dictionary
using System.ServiceModel;                          //EnpointAddress
using System.Security.Cryptography.X509Certificates;//X509 classes
using System.IO;                                    //File
using System.Net;                                   //ServicePointManager
using System.Net.Security;                          //SslPolicyErrors
using System.Xml.Serialization;                     //XmlSerializer

namespace MUISample
{
    ///<summary>
    /// This sample program shows how to interact with eMarket Market User 
    /// Interface (MUI) using web services. The program relies on the 
    /// presence of the EnergyMarketMUI interface which is generated from
    /// the eMktMUI.wsdl using the Microsoft svcutil program. 
    /// 
    /// The following command was used to generate code for this program:
    ///     svcutil eMktMUI.wsdl eMktMessages.xsd
    /// This generates the EnergyMarketMUIService.cs file which contains most
    /// of the classes used in this example.
    ///</summary>

class Program
    {
        //-----------------------------------------------------------------
        // Configurable settings are driven from a properties file and held
        // in a local properties Dictionary.
        //-----------------------------------------------------------------

        
        private const string PROPERTIES_PATH = "./MUISample.properties";
        private static Dictionary<string, string> properties;

        static void Main(string[] args)
        {
            GetPropertiesFromFile(PROPERTIES_PATH);
            try
            {
                // Setup the proxy to the web service.
                EnergyMarketMUI proxy = CreateWebServiceClient();
                // Try a simple Query and Submit type operation.
                TestGetNode(proxy);
                TestSubmitPortfolio(proxy);
                TestGetUnit(proxy);
                //TestGetFuelPriceAdjustmentUnit(proxy);
                //TestGetFuelPriceAdjustmentAttachment(proxy);
                //TestGetSubAccount(proxy);
                TestGetDRR(proxy);
            }

            catch (FaultException<www.markets.isone.com.MUI.eMkt.Messages.MUIFault> soapFault)
            {
                Console.WriteLine("\n*** Operation produced a Fault ***\n");
                // display details of soap message
                foreach (www.markets.isone.com.MUI.eMkt.Messages.TransactionVariance tv in soapFault.Detail)
                    Console.WriteLine("ErrorMessage: {0}", tv.Nodes[0].LastChild.Value);
            }
            catch (Exception e)
            {
                Console.WriteLine("\n*** Operation produced a Fault ***\n");
                Console.WriteLine(e.Message);
            }

            //Uncomment to have program exit automatically
            //Environment.Exit(0);

            //Uncomment to have program stay open until 'Enter' key is pressed.
            //If "Environment.Exit(0);" above is uncommented, will not stay open.
            Console.WriteLine("\nPress 'Enter' to exit..\n");
            Console.ReadLine();
        }

        //---------------------------------------------------------------------
        ///<summary>
        /// Executes a "GetNode" operation and prints out the results.
        ///</summary>
        //---------------------------------------------------------------------
        private static void TestGetNode(EnergyMarketMUI proxy)
        {
            Console.WriteLine("\n-----------------------");
            Console.WriteLine("  Testing GetNode");
            Console.WriteLine("\n-----------------------\n");

            // The XML structure for this message is as follows:
            //    <GetNode party="SomeParty"/>

            // Create the containing request object
            GetNodeRequest request = new GetNodeRequest();

            // Create the GetNode object which maps to the <GetNode> element.
            request.GetNode = new GetNode();
            try
            { request.GetNode.party = properties["client.party"]; }
            catch (Exception) { Console.WriteLine("Info: Unable to read MUISample.properties client.party"); }
                                                        

            // Call the service and print out the response.
            GetNodeResponse response = proxy.GetNode(request);
            int nodeCount = response.GetNodeResponse1.Length;
            Console.Write("Nodes returned: {0}. ", nodeCount);
            Console.WriteLine("Here are the first 10:\n");
            int loopCount = 0;
            foreach (NodeOut node in response.GetNodeResponse1)
            {
                Console.WriteLine("Node ID = {0,-8} Name = {1,-25} Type = {2}", 
                                   node.ID, node.name, node.type);
                loopCount++;
                if (loopCount == 10) 
                    break;
            }
        }

        //---------------------------------------------------------------------
        ///<summary>
        /// Executes a "SubmitPortfolio" operation and prints out the results.
        ///</summary>
        //---------------------------------------------------------------------
        private static void TestSubmitPortfolio(EnergyMarketMUI proxy)
        {

            Console.WriteLine("\n-----------------------");
            Console.WriteLine("  Testing SubmitPortfolio");
            Console.WriteLine("\n-----------------------\n");
            
            // The XML structure for this message is as follows:
            // <SubmitPortfolio party="SomeParty">
            //   <!-- 1 or more -->
            //   <Portfolio name="SomeName" type="TheType">
            //     <!-- 1 or more -->
            //     <Resource name="SomeResource" resourceID="TheID"/>
            //   </Portfolio>
            // </SubmitPortfolio>

            // Create the containing request object
            SubmitPortfolioRequest request = new SubmitPortfolioRequest();

            // Create the SubmitPortfolio object. This maps to the 
            // <SubmitPortfolio> element. Link to the request object.
            SubmitPortfolio submitPortfolio = new SubmitPortfolio();
            try
            { submitPortfolio.party = properties["client.party"]; }
            catch (Exception) { Console.WriteLine("Info: Unable to read MUISample.properties client.party"); }
            request.SubmitPortfolio = submitPortfolio;


            // Create the portfolio container object. It will contain 1 of the
            // <Portfolio> elements in this sample.
            PortfolioInOut[] portfolioContainer = new PortfolioInOut[1];
            submitPortfolio.Portfolio = portfolioContainer;

            // Create the portfolio object. This maps to the <Portfolio>
            // element. Assign to the portfolio container.
            PortfolioInOut portfolio = new PortfolioInOut();
            portfolio.name = properties["portfolio.name"];
            portfolio.type = PortfolioTypeEnum.Pnode;
            portfolioContainer[0] = portfolio;

            // Create the container for portfolio entries. It will contain 1 of 
            // the <Resource> elements in this sample.
            PortfolioEntry[] portfolioEntries = new PortfolioEntry[1];

            // Create the PortfolioEntry object. This maps to the <Resource>
            // element. 
            PortfolioEntry portfolioEntry = new PortfolioEntry();
            portfolioEntry.name = "LD.STEPSTON23";
            portfolioEntry.resourceID = 16184;

            // Assign the portfolio entry to the array of entries. Assign
            // the array of entries to the Portfolio object.
            portfolioEntries[0] = portfolioEntry;
            portfolio.Resource = portfolioEntries;

            // Here is the request in XML form
            Console.WriteLine("Calling SubmitPortfolio with this message:\n");
            XmlSerializer x = new XmlSerializer(request.GetType());
            x.Serialize(Console.Out, request);
            Console.WriteLine();

            // Submit the request and print out the response
            SubmitPortfolioResponse response = proxy.SubmitPortfolio(request);
            Console.WriteLine("Transaction ID = {0}", response.SubmitConfirmation.transactionId);
        }

        //---------------------------------------------------------------------
        ///<summary>
        /// Executes a "GetUnit" operation and prints out the results.
        ///</summary>
        //---------------------------------------------------------------------
        private static void TestGetUnit(EnergyMarketMUI proxy)
        {
            Console.WriteLine("\n-----------------------");
            Console.WriteLine("  Testing GetUnit");
            Console.WriteLine("\n-----------------------\n");

            // Create the containing request object
            GetUnitRequest request = new GetUnitRequest();

            // Create the GetUnit object which maps to the <GetUnit> element.
            request.GetUnit = new GetUnit();
            try
            { request.GetUnit.party = properties["client.party"]; }
            catch (Exception) { Console.WriteLine("Info: Unable to read MUISample.properties client.party"); }
            request.GetUnit.QueryFilters = new QueryByDayAndId();
            request.GetUnit.QueryFilters.Day = DateTime.Today.AddDays(1); // For Tomorrow

            // Call the service and print out the response.
            GetUnitResponse response = proxy.GetUnit(request);
            int UnitCount = response.GetUnitResponse1.Length;
            Console.Write("Units returned: {0}. ", UnitCount);
            Console.WriteLine("Here are the first 2:\n");
            int loopCount = 0;
            foreach (UnitOut unit in response.GetUnitResponse1)
            {
                Console.WriteLine("Unit ID = {0,-8} UnitShortName = {1,-25} ",
                                   unit.ID, unit.UnitShortName);
                loopCount++;
                if (loopCount == 2)
                    break;
            }
        }

        //---------------------------------------------------------------------
        ///<summary>
        /// Executes a "GetFuelPriceAdjustmentUnit" operation and prints out the results.
        ///</summary>
        //---------------------------------------------------------------------
        private static void TestGetFuelPriceAdjustmentUnit(EnergyMarketMUI proxy)
        {
            Console.WriteLine("\n-----------------------");
            Console.WriteLine("  Testing GetFuelPriceAdjustmentUnit");
            Console.WriteLine("\n-----------------------\n");

            // Create the containing request object
            GetFuelPriceAdjustmentUnitRequest request = new GetFuelPriceAdjustmentUnitRequest();

            // Create the GetFuelPriceAdjustmentUnit object which maps to the <GetFuelPriceAdjustmentUnit> element.
            request.GetFuelPriceAdjustmentUnit = new GetFuelPriceAdjustmentUnit();
            try
            { request.GetFuelPriceAdjustmentUnit.party = properties["client.party"]; }
            catch (Exception) { Console.WriteLine("Info: Unable to read MUISample.properties client.party"); }
            request.GetFuelPriceAdjustmentUnit.QueryFilters = new QueryByDayIdAndFuelIndex();
            request.GetFuelPriceAdjustmentUnit.QueryFilters.Day = DateTime.Today.AddDays(1); // For Tomorrow

            // Call the service and print out the response.
            GetFuelPriceAdjustmentUnitResponse response = proxy.GetFuelPriceAdjustmentUnit(request);
            int FPAsCount = response.GetFuelPriceAdjustmentUnitResponse1.Length;
            Console.Write("FPAs returned: {0}. ", FPAsCount);
            Console.WriteLine("Here are the first 10:\n");
            int loopCount = 0;
            foreach (FuelPriceAdjustmentUnit fpa in response.GetFuelPriceAdjustmentUnitResponse1)
            {
                Console.WriteLine("Unit ID = {0,-8} UnitLongName = {1,-25} ",
                                   fpa.ID, fpa.UnitLongName);
                loopCount++;
                if (loopCount == 10)
                    break;
            }
        }

        //---------------------------------------------------------------------
        ///<summary>
        /// Executes a "GetFuelPriceAdjustmentAttachment" operation and prints out the results.
        ///</summary>
        //---------------------------------------------------------------------
        private static void TestGetFuelPriceAdjustmentAttachment(EnergyMarketMUI proxy)
        {
            Console.WriteLine("\n-----------------------");
            Console.WriteLine("  Testing GetFuelPriceAdjustmentUnit");
            Console.WriteLine("\n-----------------------\n");

            // Create the containing request object
            GetFuelPriceAdjustmentAttachmentRequest request = new GetFuelPriceAdjustmentAttachmentRequest();

            // Create the GetFuelPriceAdjustmentAttachment object which maps to the <GetFuelPriceAdjustmentAttachment> element.
            request.GetFuelPriceAdjustmentAttachment = new GetFuelPriceAdjustmentAttachment();
            try
            { request.GetFuelPriceAdjustmentAttachment.party = properties["client.party"]; }
            catch (Exception) { Console.WriteLine("Info: Unable to read MUISample.properties client.party"); }
            request.GetFuelPriceAdjustmentAttachment.QueryFilters = new QueryByFPAId();
            request.GetFuelPriceAdjustmentAttachment.QueryFilters.FPAID = 123456;

            // Call the service and print out the response.
            GetFuelPriceAdjustmentAttachmentResponse response = proxy.GetFuelPriceAdjustmentAttachment(request);
            int FPAsCount = response.GetFuelPriceAdjustmentAttachmentResponse1.Length;
            Console.Write("FPA Attachments returned: {0}. ", FPAsCount);
            Console.WriteLine("Here are the first 10:\n");
            int loopCount = 0;
            foreach (FuelPriceAdjustmentAttachmentOut att in response.GetFuelPriceAdjustmentAttachmentResponse1)
            {
                Console.WriteLine("FPA ID = {0,-8} FileName = {1,-25} UploadedBy = {2,-25}",
                                   att.FPAID, att.Attachments[0].FileName, att.Attachments[0].UploadedBy);
                loopCount++;
                if (loopCount == 10)
                    break;
            }
        }

        //---------------------------------------------------------------------
        ///<summary>
        /// Executes a "GetSubAccount" operation and prints out the results.
        ///</summary>
        //---------------------------------------------------------------------
        private static void TestGetSubAccount(EnergyMarketMUI proxy)
        {
            Console.WriteLine("\n-----------------------");
            Console.WriteLine("  Testing GetSubAccount");
            Console.WriteLine("\n-----------------------\n");

            // The XML structure for this message is as follows:
            //    <GetNode party="SomeParty"/>

            // Create the containing request object
            GetSubAccountRequest request = new GetSubAccountRequest();

            // Create the GetSubAccount object which maps to the <GetSubAccount> element.
            request.GetSubAccount = new GetSubAccount();
            try
            { request.GetSubAccount.party = properties["client.party"]; }
            catch (Exception) { Console.WriteLine("Info: Unable to read MUISample.properties client.party"); }
            request.GetSubAccount.Day = DateTime.Today.AddDays(1); // Query for tomorrow

            // Call the service and print out the response.
            GetSubAccountResponse response = proxy.GetSubAccount(request);
            int nodeCount = response.GetSubAccountResponse1.Length;
            Console.Write("SubAccounts returned: {0}. ", nodeCount);
            Console.WriteLine("Here are the first 10:\n");
            int loopCount = 0;
            foreach (SubAccountOut sa in response.GetSubAccountResponse1)
            {
                Console.WriteLine("SubaccountName = {0,-25} Default = {1,-8} Terminated = {2,-8}",
                                   sa.Name, sa.Default, sa.Terminated);
                loopCount++;
                if (loopCount == 10)
                    break;
            }
        }

        //---------------------------------------------------------------------
        ///<summary>
        /// Executes a "GetDRR" operation and prints out the results.
        ///</summary>
        //---------------------------------------------------------------------
        private static void TestGetDRR(EnergyMarketMUI proxy)
        {
            Console.WriteLine("\n-----------------------");
            Console.WriteLine("  Testing GetDRR");
            Console.WriteLine("\n-----------------------\n");

            // Create the containing request object
            GetDRRRequest request = new GetDRRRequest();

            // Create the GetDRR object which maps to the <GetDRR> element.
            request.GetDRR = new GetDRR();
            try
            { request.GetDRR.party = properties["client.party"]; }
            catch (Exception) { Console.WriteLine("Info: Unable to read MUISample.properties client.party"); }
            request.GetDRR.QueryFilters = new QueryByDayAndId();
            request.GetDRR.QueryFilters.Day = DateTime.Today.AddDays(1); // For Tomorrow

            // Call the service and print out the response.
            GetDRRResponse response = proxy.GetDRR(request);
            int DRRCount = response.GetDRRResponse1.Length;
            Console.Write("DRRs returned: {0}. ", DRRCount);
            Console.WriteLine("Here are the first 2:\n");
            int loopCount = 0;
            foreach (DRROut drr in response.GetDRRResponse1)
            {
                Console.WriteLine("DRR ID = {0,-8} DRRShortName = {1,-25} ",
                                   drr.ID, drr.DRRShortName);
                loopCount++;
                if (loopCount == 2)
                    break;
            }
        }

        //---------------------------------------------------------------------
        ///<summary>
        /// Setup the endpoints and certificates and return the proxy object
        /// for the web service.
        ///</summary>
        //---------------------------------------------------------------------
        private static EnergyMarketMUI CreateWebServiceClient()
        {
            EndpointAddress address = null;
            try{ address = new EndpointAddress(properties["endpoint.link"]); }
            catch (Exception ex)
            { throw new Exception("Unable to create EndpointAddress.  \n" +
                                  "Check MUISample.properties endpoint.link \n" + 
                                  ex.Message );}

            // The client will use a 509 certificate. 
            X509Certificate2 clientKey = null;
            try
            {  clientKey = new X509Certificate2(
                           properties["client.key.file"],
                           properties["client.key.pswd"]);}
            catch (Exception ex)
            {  throw new Exception("Unable to create ClientCertificate. \n" +
                                    "Check MUISample.properties client.key.file and client.key.pswd \n" + 
                                    ex.Message );}

            // And the server must have trusted certificates.
            X509Certificate2 serverCertifications = null;
            try
            { serverCertifications = new X509Certificate2(properties["srvr.trust.crt"]); }
            catch (Exception ex)
            {
                throw new Exception("Unable to create TrustedServerCertificate. \n" +
                                       "Check MUISample.properties srvr.trust.crt \n" +
                                       ex.Message);
            }


            //-----------------------------------------------------------------
            // Establish a proxy for the service
            //-----------------------------------------------------------------

            EnergyMarketMUI proxy = null;
            try
            {
                proxy = GetService(
                        address,
                        clientKey,
                        serverCertifications);
            }
            catch (Exception)
            {
                Console.WriteLine("Unable to create EnergyMarketMUI proxy");
                throw;
            }

            return proxy;
        }

        //---------------------------------------------------------------------
        ///<summary>
        /// Establishes a connection to the specified service. This can also be
        /// specified in a WCF configuration file. 
        /// See http://msdn.microsoft.com/en-us/library/ms735103.aspx
        ///</summary>
        //---------------------------------------------------------------------
        private static EnergyMarketMUI GetService(EndpointAddress address, X509Certificate2 clientCert, X509Certificate2 serverTrust)
        {
            // Binding for proxy
            BasicHttpBinding binding = new BasicHttpBinding();

            // set security mode to Transport
            binding.Security.Mode = BasicHttpSecurityMode.Transport;
            binding.MessageEncoding = WSMessageEncoding.Text;
            
            // set credential type to require certificates
            binding.Security.Transport.ClientCredentialType = HttpClientCredentialType.Certificate;
            

            // create channel using binding and address
            ChannelFactory<EnergyMarketMUI> factory
                = new ChannelFactory<EnergyMarketMUI>(binding, address);

            // assign client side certificate
            factory.Credentials.ClientCertificate.Certificate = clientCert;

            // assign server trusted certificates
            factory.Credentials.ServiceCertificate.DefaultCertificate = serverTrust;

            ServicePointManager.ServerCertificateValidationCallback += delegate(object sender1, X509Certificate certificate,
                                                X509Chain chain, SslPolicyErrors sslPolicyErrors) { return true; };
            ServicePointManager.SecurityProtocol = SecurityProtocolType.Tls12;

            // create proxy connection to the server
            return factory.CreateChannel();
        }

        //---------------------------------------------------------------------
        ///<summary>
        /// Read properties in from file and load into the Dictionary.
        ///</summary>
        //---------------------------------------------------------------------
        private static void GetPropertiesFromFile(string address)
        {
            properties = new Dictionary<string, string>(10);
            string cleanRow;
            foreach (string row in File.ReadAllLines(address))
            {
                cleanRow = row.Trim();
                if ((!cleanRow.StartsWith("#")) & (cleanRow.Length > 0))
                    properties.Add(cleanRow.Split('=')[0].Trim(), 
                                   cleanRow.Split('=')[1].Trim());
            }
        }


    }
}
